<?php

namespace Drupal\discord_notifications\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\State\StateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Discord Notifications settings for this site.
 */
class DiscordNotificationsSettingsForm extends ConfigFormBase {

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Constructs a new DiscordNotificationsSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state) {
    parent::__construct($config_factory);
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'discord_notifications_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['discord_notifications.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('discord_notifications.settings');

    $form['webhook_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Discord Webhook URL'),
      '#default_value' => $config->get('webhook_url'),
      '#description' => $this->t('Enter the Discord webhook URL for sending notifications.'),
      '#required' => TRUE,
    ];

    $form['discord_alerts'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Discord Alerts'),
    ];

    $form['discord_alerts']['use_here'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use @here mention'),
      '#description' => $this->t('If checked, notifications will include an @here mention.'),
      '#default_value' => $config->get('use_here') ?: FALSE,
    ];

    $form['discord_alerts']['use_everyone'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use @everyone mention'),
      '#description' => $this->t('If checked, notifications will include an @everyone mention.'),
      '#default_value' => $config->get('use_everyone') ?: FALSE,
    ];

    $form['notifications'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Notification Settings'),
    ];

    $form['notifications']['content_notifications'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Content Notifications'),
      '#options' => [
        'node_create' => $this->t('New content created'),
        'node_update' => $this->t('Content updated'),
        'node_delete' => $this->t('Content deleted'),
      ],
      '#default_value' => $config->get('content_notifications') ?: [],
    ];

    $form['notifications']['user_notifications'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('User Notifications'),
      '#options' => [
        'user_register' => $this->t('New user registered'),
        'user_login' => $this->t('User login'),
        'user_blocked' => $this->t('User blocked'),
      ],
      '#default_value' => $config->get('user_notifications') ?: [],
    ];

    $form['notifications']['system_notifications'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('System Notifications'),
      '#options' => [
        'update_available' => $this->t('Drupal core or module updates available'),
        'cron_run' => $this->t('Cron run completed'),
      ],
      '#default_value' => $config->get('system_notifications') ?: [],
    ];

    $form['notifications']['security_notifications'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Security Notifications'),
      '#options' => [
        'failed_login' => $this->t('Failed login attempts'),
        'password_reset' => $this->t('Password reset requests'),
      ],
      '#default_value' => $config->get('security_notifications') ?: [],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('discord_notifications.settings')
      ->set('webhook_url', $form_state->getValue('webhook_url'))
      ->set('use_here', $form_state->getValue('use_here'))
      ->set('use_everyone', $form_state->getValue('use_everyone'))
      ->set('content_notifications', array_filter($form_state->getValue('content_notifications')))
      ->set('user_notifications', array_filter($form_state->getValue('user_notifications')))
      ->set('system_notifications', array_filter($form_state->getValue('system_notifications')))
      ->set('security_notifications', array_filter($form_state->getValue('security_notifications')))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
