<?php

namespace Drupal\discord_notifications\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\user\UserInterface;
use GuzzleHttp\ClientInterface;

/**
 * Service for sending Discord notifications.
 */
class DiscordNotificationService {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Constructs a new DiscordNotificationService object.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   */
  public function __construct(
    ClientInterface $http_client,
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    AccountProxyInterface $current_user,
  ) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $this->currentUser = $current_user;
  }

  /**
   * Sends a notification to Discord.
   *
   * @param string $message
   *   The message to send.
   * @param array $embed
   *   Optional embed data to send with the message.
   */
  public function sendNotification($message, array $embed = []) {
    $config = $this->configFactory->get('discord_notifications.settings');
    $webhook_url = $config->get('webhook_url');

    if (empty($webhook_url)) {
      $this->loggerFactory->get('discord_notifications')->error('Discord webhook URL is not configured.');
      return;
    }

    $use_here = $config->get('use_here');
    $use_everyone = $config->get('use_everyone');

    if ($use_here) {
      $message = '@here ' . $message;
    }
    elseif ($use_everyone) {
      $message = '@everyone ' . $message;
    }

    $data = [
      'content' => $message,
    ];

    if (!empty($embed)) {
      $data['embeds'] = [$embed];
    }

    try {
      $this->httpClient->post($webhook_url, [
        'json' => $data,
      ]);
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('discord_notifications')
        ->error('Failed to send Discord notification: @error', ['@error' => $e->getMessage()]);
    }
  }

  /**
   * Handles content notifications.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   * @param string $operation
   *   The operation performed (create, update, delete).
   */
  public function handleContentNotification(EntityInterface $entity, $operation) {
    $config = $this->configFactory->get('discord_notifications.settings');
    $content_notifications = $config->get('content_notifications');

    if (!in_array("node_{$operation}", $content_notifications)) {
      return;
    }

    $colors = [
      'create' => 0x00FF00,
      'update' => 0xFFFF00,
      'delete' => 0xFF0000,
    ];

    $embed = [
      'title' => ucfirst($operation) . 'd Content',
      'description' => "{$entity->getTitle()} (ID: {$entity->id()})",
      'color' => $colors[$operation],
      'footer' => [
        'text' => "Action performed by: " . $this->currentUser->getDisplayName(),
      ],
      'timestamp' => date('c'),
    ];

    $this->sendNotification('', $embed);
  }

  /**
   * Handles user notifications.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param string $operation
   *   The operation performed (register, login, block).
   */
  public function handleUserNotification(UserInterface $user, $operation) {
    $config = $this->configFactory->get('discord_notifications.settings');
    $user_notifications = $config->get('user_notifications');

    if (!in_array("user_{$operation}", $user_notifications)) {
      return;
    }

    $colors = [
      'register' => 0x00FF00,
      'login' => 0x0000FF,
      'block' => 0xFF0000,
    ];

    $embed = [
      'title' => 'User ' . ucfirst($operation),
      'description' => "{$user->getDisplayName()} (ID: {$user->id()})",
      'color' => $colors[$operation],
      'footer' => [
        'text' => "Action performed by: " . $this->currentUser->getDisplayName(),
      ],
      'timestamp' => date('c'),
    ];

    $this->sendNotification('', $embed);
  }

  /**
   * Handles system notifications.
   *
   * @param string $type
   *   The type of system notification (update_available, cron_run).
   * @param string $details
   *   Additional details about the notification.
   */
  public function handleSystemNotification($type, $details = '') {
    $config = $this->configFactory->get('discord_notifications.settings');
    $system_notifications = $config->get('system_notifications');

    if (!in_array($type, $system_notifications)) {
      return;
    }

    $embed = [
      'title' => 'System Notification',
      'description' => ucfirst(str_replace('_', ' ', $type)),
      'fields' => [
        [
          'name' => 'Details',
          'value' => $details,
        ],
      ],
      'color' => 0x0000FF,
      'timestamp' => date('c'),
    ];

    $this->sendNotification('', $embed);
  }

  /**
   * Handles security notifications.
   *
   * @param string $type
   *   The type of security notification (failed_login, password_reset).
   * @param string $details
   *   Additional details about the notification.
   */
  public function handleSecurityNotification($type, $details = '') {
    $config = $this->configFactory->get('discord_notifications.settings');
    $security_notifications = $config->get('security_notifications');

    if (!in_array($type, $security_notifications)) {
      return;
    }

    $embed = [
      'title' => 'Security Alert',
      'description' => ucfirst(str_replace('_', ' ', $type)),
      'fields' => [
        [
          'name' => 'Details',
          'value' => $details,
        ],
      ],
      'color' => 0xFF0000,
      'timestamp' => date('c'),
    ];

    $this->sendNotification('', $embed);
  }

}
